=== modified file 'NEWS'
--- NEWS	2006-07-03 13:48:52 +0000
+++ NEWS	2006-07-04 15:04:11 +0000
@@ -69,6 +69,9 @@
    * 'branches.conf' has been changed to 'locations.conf', since it can apply
      to more locations than just branch locations.
      (Aaron Bentley)
+   
+   * 'bzr whoami' can now be used to set your identity from the command line,
+     for a branch or globally.  (Robey Pointer)
 
    * 'bzr checkout' now aliased to 'bzr co', and 'bzr annotate' to 'bzr ann'.
      (Michael Ellerman)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-07-03 13:35:59 +0000
+++ bzrlib/builtins.py	2006-07-04 15:04:11 +0000
@@ -1788,19 +1788,41 @@
 
 
 class cmd_whoami(Command):
-    """Show bzr user id."""
-    takes_options = ['email']
+    """Show or set bzr user id.
+    
+    examples:
+        bzr whoami --email
+        bzr whoami 'Frank Chu <fchu@example.com>'
+    """
+    takes_options = [ Option('email',
+                             help='display email address only'),
+                      Option('branch',
+                             help='set identity for the current branch instead of '
+                                  'globally'),
+                    ]
+    takes_args = ['name?']
+    encoding_type = 'replace'
     
     @display_command
-    def run(self, email=False):
-        try:
-            c = WorkingTree.open_containing(u'.')[0].branch.get_config()
-        except NotBranchError:
+    def run(self, email=False, branch=False, name=None):
+        if name is None:
+            # use branch if we're inside one; otherwise global config
+            try:
+                c = Branch.open_containing('.')[0].get_config()
+            except NotBranchError:
+                c = config.GlobalConfig()
+            if email:
+                self.outf.write(c.user_email() + '\n')
+            else:
+                self.outf.write(c.username() + '\n')
+            return
+
+        # use global config unless --branch given
+        if branch:
+            c = Branch.open_containing('.')[0].get_config()
+        else:
             c = config.GlobalConfig()
-        if email:
-            print c.user_email()
-        else:
-            print c.username()
+        c.set_user_option('email', name)
 
 
 class cmd_nick(Command):

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2006-07-02 01:21:07 +0000
+++ bzrlib/config.py	2006-07-04 15:04:11 +0000
@@ -337,6 +337,19 @@
     def __init__(self):
         super(GlobalConfig, self).__init__(config_filename)
 
+    def set_user_option(self, option, value):
+        """Save option and its value in the configuration."""
+        # FIXME: RBC 20051029 This should refresh the parser and also take a
+        # file lock on bazaar.conf.
+        conf_dir = os.path.dirname(self._get_filename())
+        ensure_config_dir_exists(conf_dir)
+        if 'DEFAULT' not in self._get_parser():
+            self._get_parser()['DEFAULT'] = {}
+        self._get_parser()['DEFAULT'][option] = value
+        f = open(self._get_filename(), 'wb')
+        self._get_parser().write(f)
+        f.close()
+
 
 class LocationConfig(IniBasedConfig):
     """A configuration object that gives the policy for a location."""
@@ -611,13 +624,25 @@
         uid = os.getuid()
         w = pwd.getpwuid(uid)
 
-        try:
-            gecos = w.pw_gecos.decode(bzrlib.user_encoding)
-            username = w.pw_name.decode(bzrlib.user_encoding)
-        except UnicodeDecodeError:
-            # We're using pwd, therefore we're on Unix, so /etc/passwd is ok.
-            raise errors.BzrError("Can't decode username in " \
-                    "/etc/passwd as %s." % bzrlib.user_encoding)
+        # we try utf-8 first, because on many variants (like Linux),
+        # /etc/passwd "should" be in utf-8, and because it's unlikely to give
+        # false positives.  (many users will have their user encoding set to
+        # latin-1, which cannot raise UnicodeError.)
+        try:
+            gecos = w.pw_gecos.decode('utf-8')
+            encoding = 'utf-8'
+        except UnicodeError:
+            try:
+                gecos = w.pw_gecos.decode(bzrlib.user_encoding)
+                encoding = bzrlib.user_encoding
+            except UnicodeError:
+                raise errors.BzrCommandError('Unable to determine your name.  '
+                   'Use "bzr whoami" to set it.')
+        try:
+            username = w.pw_name.decode(encoding)
+        except UnicodeError:
+            raise errors.BzrCommandError('Unable to determine your name.  '
+                'Use "bzr whoami" to set it.')
 
         comma = gecos.find(',')
         if comma == -1:

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-07-03 08:12:23 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-07-04 15:04:11 +0000
@@ -84,6 +84,7 @@
                      'bzrlib.tests.blackbox.test_update',
                      'bzrlib.tests.blackbox.test_upgrade',
                      'bzrlib.tests.blackbox.test_versioning',
+                     'bzrlib.tests.blackbox.test_whoami',
                      ]
     test_encodings = [
         'bzrlib.tests.blackbox.test_non_ascii',

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2006-07-03 13:48:52 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-07-04 15:04:11 +0000
@@ -57,36 +57,6 @@
 
 class TestCommands(ExternalBase):
 
-    def test_whoami(self):
-        # this should always identify something, if only "john@localhost"
-        self.runbzr("whoami")
-        self.runbzr("whoami --email")
-
-        self.assertEquals(self.runbzr("whoami --email",
-                                      backtick=True).count('@'), 1)
-        
-    def test_whoami_branch(self):
-        """branch specific user identity works."""
-        self.runbzr('init')
-        b = bzrlib.branch.Branch.open('.')
-        b.control_files.put_utf8('email', 'Branch Identity <branch@identi.ty>')
-        bzr_email = os.environ.get('BZREMAIL')
-        if bzr_email is not None:
-            del os.environ['BZREMAIL']
-        whoami = self.runbzr("whoami",backtick=True)
-        whoami_email = self.runbzr("whoami --email",backtick=True)
-        self.assertTrue(whoami.startswith('Branch Identity <branch@identi.ty>'))
-        self.assertTrue(whoami_email.startswith('branch@identi.ty'))
-        # Verify that the environment variable overrides the value 
-        # in the file
-        os.environ['BZREMAIL'] = 'Different ID <other@environ.ment>'
-        whoami = self.runbzr("whoami",backtick=True)
-        whoami_email = self.runbzr("whoami --email",backtick=True)
-        self.assertTrue(whoami.startswith('Different ID <other@environ.ment>'))
-        self.assertTrue(whoami_email.startswith('other@environ.ment'))
-        if bzr_email is not None:
-            os.environ['BZREMAIL'] = bzr_email
-
     def test_nick_command(self):
         """bzr nick for viewing, setting nicknames"""
         os.mkdir('me.dev')

=== added file 'bzrlib/tests/blackbox/test_whoami.py'
--- bzrlib/tests/blackbox/test_whoami.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_whoami.py	2006-07-03 08:52:44 +0000
@@ -0,0 +1,95 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr whoami."""
+
+import os
+
+import bzrlib
+from bzrlib.branch import Branch
+from bzrlib.tests.blackbox import ExternalBase
+
+
+class TestWhoami(ExternalBase):
+
+    def test_whoami(self):
+        # this should always identify something, if only "john@localhost"
+        out = self.run_bzr("whoami")[0]
+        self.assertTrue(len(out) > 0)
+        self.assertEquals(1, out.count('@'))
+
+        out = self.run_bzr("whoami", "--email")[0]
+        self.assertTrue(len(out) > 0)
+        self.assertEquals(1, out.count('@'))
+        
+    def test_whoami_branch(self):
+        """branch specific user identity works."""
+        wt = self.make_branch_and_tree('.')
+        b = bzrlib.branch.Branch.open('.')
+        b.get_config().set_user_option('email', 'Branch Identity <branch@identi.ty>')
+        bzr_email = os.environ.get('BZREMAIL')
+        if bzr_email is not None:
+            del os.environ['BZREMAIL']
+        try:
+            whoami = self.run_bzr("whoami")[0]
+            self.assertEquals('Branch Identity <branch@identi.ty>\n', whoami)
+            whoami_email = self.run_bzr("whoami", "--email")[0]
+            self.assertEquals('branch@identi.ty\n', whoami_email)
+
+            # Verify that the environment variable overrides the value 
+            # in the file
+            os.environ['BZREMAIL'] = 'Different ID <other@environ.ment>'
+            whoami = self.run_bzr("whoami")[0]
+            self.assertEquals('Different ID <other@environ.ment>\n', whoami)
+            whoami_email = self.run_bzr("whoami", "--email")[0]
+            self.assertEquals('other@environ.ment\n', whoami_email)
+        finally:
+            if bzr_email is not None:
+                os.environ['BZREMAIL'] = bzr_email
+
+    def test_whoami_utf8(self):
+        """verify that an identity can be in utf-8."""
+        wt = self.make_branch_and_tree('.')
+        self.run_bzr('whoami', u'Branch Identity \u20ac <branch@identi.ty>', encoding='utf-8')
+        bzr_email = os.environ.get('BZREMAIL')
+        if bzr_email is not None:
+            del os.environ['BZREMAIL']
+        try:
+            whoami = self.run_bzr("whoami", encoding='utf-8')[0]
+            self.assertEquals('Branch Identity \xe2\x82\xac <branch@identi.ty>\n', whoami)
+            whoami_email = self.run_bzr("whoami", "--email", encoding='utf-8')[0]
+            self.assertEquals('branch@identi.ty\n', whoami_email)
+        finally:
+            if bzr_email is not None:
+                os.environ['BZREMAIL'] = bzr_email
+
+    def test_whoami_ascii(self):
+        """verify that whoami doesn't totally break when in utf-8, using an ascii encoding."""
+        wt = self.make_branch_and_tree('.')
+        b = bzrlib.branch.Branch.open('.')
+        b.get_config().set_user_option('email', u'Branch Identity \u20ac <branch@identi.ty>')
+        bzr_email = os.environ.get('BZREMAIL')
+        if bzr_email is not None:
+            del os.environ['BZREMAIL']
+        try:
+            whoami = self.run_bzr("whoami", encoding='ascii')[0]
+            self.assertEquals('Branch Identity ? <branch@identi.ty>\n', whoami)
+            whoami_email = self.run_bzr("whoami", "--email", encoding='ascii')[0]
+            self.assertEquals('branch@identi.ty\n', whoami_email)
+        finally:
+            if bzr_email is not None:
+                os.environ['BZREMAIL'] = bzr_email

=== modified file 'doc/README.1st'
--- doc/README.1st	2006-03-14 16:57:35 +0000
+++ doc/README.1st	2006-06-29 23:25:48 +0000
@@ -1,6 +1,6 @@
 These documents originally came from the Bazaar-NG wiki hosted at
-http://bazaar-vcs.org. These documents, though maintained, These documenets
-can fall behind when Bazaar-NG is under heavy development.
+http://bazaar-vcs.org. These documents, though maintained, can fall behind
+when Bazaar-NG is under heavy development.
 
 The documents on the wiki take precedence in the event of a discrepancy
 between what is documented here and what is documented on the wiki.

=== modified file 'doc/setting_up_email.txt'
--- doc/setting_up_email.txt	2006-06-16 15:59:24 +0000
+++ doc/setting_up_email.txt	2006-06-29 23:26:31 +0000
@@ -31,14 +31,25 @@
 
     % bzr whoami
     Joe Cool <joe@example.com>
+    
+Setting email via the 'whoami' command
+======================================
+You can use the whoami command to set your email globally::
+
+    % bzr whoami 'Joe Cool <joe@example.com>'
+
+or only for the current branch::
+
+    % bzr whoami --branch 'Joe Cool <joe@example.com>'
+
+These modify your global bazaar.conf or branch branch.conf file, respectively.
 
 Setting email via default ini file
 ==================================
-The first method is using the default ini file.  To use the default ini
-method, create the file `$HOME/.bazaar/bazaar.conf` (on Windows this is
-`%APPDATA%\bazaar\2.0\bazaar.conf`) and set an email address as shown
-below.  Please note that the word DEFAULT is case sensitive, and must be in
-upper-case.::
+To use the default ini file, create the file `$HOME/.bazaar/bazaar.conf` (on
+Windows this is `%APPDATA%\bazaar\2.0\bazaar.conf`) and set an email address
+as shown below.  Please note that the word DEFAULT is case sensitive, and
+must be in upper-case.::
 
     [DEFAULT]
     email=Your Name <name@isp.com>

=== modified file 'doc/tutorial.txt'
--- doc/tutorial.txt	2006-06-17 19:01:15 +0000
+++ doc/tutorial.txt	2006-06-29 23:27:05 +0000
@@ -117,6 +117,16 @@
 address by looking up your username and hostname. If you don't like the
 guess that Bazaar-NG makes, then three options exist:
 
+ 1. Set an email address via ``bzr whoami``.  This is the simplest way.
+ To set a global identity, use::
+ 
+     % bzr whoami 'Your Name <email@example.com>'
+ 
+ If you'd like to use a different address for a specific branch, enter
+ the branch folder and use::
+ 
+     % bzr whoami --branch 'Your Name <email@example.com>'
+ 
  1. Setting the email address in the
  ``~/.bazaar/bazaar.conf`` by adding the following lines. Please note that
  ``[DEFAULT]`` is case sensitive::
@@ -124,9 +134,9 @@
     [DEFAULT]
     email= Your Name <email@isp.com>
  
- 1. Override the previous setting on a
- branch by branch basis by creating a branch section in
- ``~/.bazaar/locations.conf`` by adding the following lines::
+ As above, you can override this settings on a branch by branch basis by
+ creating a branch section in ``~/.bazaar/locations.conf`` and adding the
+ following lines::
 
     [/the/directory/to/the/branch]
     email=Your Name <email@isp.com>

